@@ -7,6 +7,7 @@
  *
  * @author John Levon
  * @author Philippe Elie
+ * @Modifications Daniel Hansel
  * Modified by Aravind Menon for Xen
  * These modifications are:
  * Copyright (C) 2005 Hewlett-Packard Co.
@@ -17,6 +18,7 @@
 #include "oprofiled.h"
 #include "opd_stats.h"
 #include "opd_sfile.h"
+#include "opd_pipe.h"
 #include "opd_kernel.h"
 #include "opd_trans.h"
 #include "opd_anon.h"
@@ -33,17 +35,26 @@
 #include <fcntl.h>
 #include <stdio.h>
 #include <errno.h>
+#include <limits.h>
 #include <stdlib.h>
+#include <sys/time.h>
+#include <wait.h>
+#include <string.h>
 
 size_t kernel_pointer_size;
 
 static fd_t devfd;
 static char * sbuf;
 static size_t s_buf_bytesize;
+extern char * session_dir;
+static char start_time_str[32];
+static int jit_conversion_running;
 
 static void opd_sighup(void);
 static void opd_alarm(void);
 static void opd_sigterm(void);
+static void opd_sigchild(void);
+static void opd_do_jitdumps(void);
 
 /**
  * opd_open_files - open necessary files
@@ -64,7 +75,7 @@ static void opd_open_files(void)
 	}
 
 	/* give output before re-opening stdout as the logfile */
-	printf("Using log file " OP_LOG_FILE "\n");
+	printf("Using log file %s\n", op_log_file);
 
 	/* set up logfile */
 	close(0);
@@ -76,6 +87,7 @@ static void opd_open_files(void)
 	}
 
 	opd_open_logfile();
+	opd_create_pipe();
 
 	printf("oprofiled started %s", op_get_time());
 	printf("kernel pointer size: %lu\n",
@@ -90,12 +102,12 @@ static void complete_dump(void)
 	FILE * status_file;
 
 retry:
-	status_file = fopen(OP_DUMP_STATUS, "w");
+	status_file = fopen(op_dump_status, "w");
 
 	if (!status_file && errno == EMFILE) {
 		if (sfile_lru_clear()) {
 			printf("LRU cleared but file open fails for %s.\n",
-			       OP_DUMP_STATUS);
+			       op_dump_status);
 			abort();
 		}
 		goto retry;
@@ -133,6 +145,48 @@ static void opd_do_samples(char const * opd_buf, ssize_t count)
 	complete_dump();
 }
  
+static void opd_do_jitdumps(void)
+{ 
+	pid_t childpid;
+	int arg_num;
+	unsigned long long end_time = 0ULL;
+	struct timeval tv;
+	char end_time_str[32];
+	char opjitconv_path[PATH_MAX + 1];
+	char * exec_args[6];
+
+	if (jit_conversion_running)
+		return;
+	jit_conversion_running = 1;
+
+	childpid = fork();
+	switch (childpid) {
+		case -1:
+			perror("Error forking JIT dump process!");
+			break;
+		case 0:
+			gettimeofday(&tv, NULL);
+			end_time = tv.tv_sec;
+			sprintf(end_time_str, "%llu", end_time);
+			sprintf(opjitconv_path, "%s/%s", OP_BINDIR, "opjitconv");
+			arg_num = 0;
+			exec_args[arg_num++] = opjitconv_path;
+			if (vmisc)
+				exec_args[arg_num++] = "-d";
+			exec_args[arg_num++] = session_dir;
+			exec_args[arg_num++] = start_time_str;
+			exec_args[arg_num++] = end_time_str;
+			exec_args[arg_num] = (char *) NULL;
+			execvp("opjitconv", exec_args);
+			fprintf(stderr, "Failed to exec %s: %s\n",
+			        exec_args[0], strerror(errno));
+			/* We don't want any cleanup in the child */
+			_exit(EXIT_FAILURE);
+		default:
+			break;
+	} 
+
+} 
 
 /**
  * opd_do_read - enter processing loop
@@ -144,6 +198,8 @@ static void opd_do_samples(char const * opd_buf, ssize_t count)
  */
 static void opd_do_read(char * buf, size_t size)
 {
+	opd_open_pipe();
+
 	while (1) {
 		ssize_t count = -1;
 
@@ -167,6 +223,9 @@ static void opd_do_read(char * buf, size_t size)
 			if (signal_term)
 				opd_sigterm();
 
+			if (signal_child)
+				opd_sigchild();
+
 			if (signal_usr1) {
 				signal_usr1 = 0;
 				perfmon_start();
@@ -176,10 +235,17 @@ static void opd_do_read(char * buf, size_t size)
 				signal_usr2 = 0;
 				perfmon_stop();
 			}
+
+			if (is_jitconv_requested()) {
+				verbprintf(vmisc, "Start opjitconv was triggered\n");
+				opd_do_jitdumps();
+			}
 		}
 
 		opd_do_samples(buf, count);
 	}
+	
+	opd_close_pipe();
 }
 
 
@@ -207,22 +273,39 @@ static void opd_sighup(void)
 static void clean_exit(void)
 {
 	perfmon_exit();
-	unlink(OP_LOCK_FILE);
+	unlink(op_lock_file);
 }
 
 
 static void opd_sigterm(void)
 {
+	opd_do_jitdumps();
 	opd_print_stats();
 	printf("oprofiled stopped %s", op_get_time());
 	exit(EXIT_FAILURE);
 }
- 
 
+/* SIGCHLD received from JIT dump child process. */
+static void opd_sigchild(void)
+{
+	int child_status;
+	wait(&child_status);
+	jit_conversion_running = 0;
+	if (WIFEXITED(child_status) && (!WEXITSTATUS(child_status))) {
+		verbprintf(vmisc, "JIT dump processing complete.\n");
+	} else {
+		printf("JIT dump processing exited abnormally: %d\n",
+		       WEXITSTATUS(child_status));
+	}
+
+}
+ 
 static void opd_26_init(void)
 {
 	size_t i;
 	size_t opd_buf_size;
+	unsigned long long start_time = 0ULL;
+	struct timeval tv;
 
 	opd_create_vmlinux(vmlinux, kernel_range);
 	opd_create_xen(xenimage, xen_range);
@@ -251,13 +334,19 @@ static void opd_26_init(void)
 		perror("oprofiled: couldn't set exit cleanup: ");
 		exit(EXIT_FAILURE);
 	}
+
+	/* trigger kernel module setup before returning control to opcontrol */
+	opd_open_files();
+	gettimeofday(&tv, NULL);
+	start_time = 0ULL;
+	start_time = tv.tv_sec;
+	sprintf(start_time_str, "%llu", start_time);
+		  
 }
 
 
 static void opd_26_start(void)
 {
-	opd_open_files();
-
 	/* simple sleep-then-process loop */
 	opd_do_read(sbuf, s_buf_bytesize);
 }
