@@ -143,7 +143,7 @@ retry:
 	}
 
 	fill_header(odb_get_data(&sfile->sample_file), counter, 0, 0,
-	            image->kernel, 0, image->mtime);
+		    image->kernel, 0, 0, 0, image->mtime);
 
 out:
 	free(mangled);
