@@ -42,6 +42,7 @@
 #include "op_cpufreq.h"
 #include "op_alloc_counter.h"
 #include "oprof_start_util.h"
+#include "file_manip.h"
 
 #include "op_hw_config.h"
 
@@ -139,9 +140,17 @@ oprof_start::oprof_start()
 	if (is_26) {
 		note_table_size_edit->hide();
 		note_table_size_label->hide();
-		// FIXME: can adapt to 2.6 ...
-		buffer_size_edit->hide();
-		buffer_size_label->hide();
+		if (!op_file_readable("/dev/oprofile/backtrace_depth")) {
+			callgraph_depth_label->hide();
+			callgraph_depth_edit->hide();
+		}
+	} else {
+		callgraph_depth_label->hide();
+		callgraph_depth_edit->hide();
+		buffer_watershed_label->hide();
+		buffer_watershed_edit->hide();
+		cpu_buffer_size_label->hide();
+		cpu_buffer_size_edit->hide();
 	}
 
 	// setup the configuration page.
@@ -150,7 +159,10 @@ oprof_start::oprof_start()
 	no_vmlinux->setChecked(config.no_kernel);
 
 	buffer_size_edit->setText(QString().setNum(config.buffer_size));
+	buffer_watershed_edit->setText(QString().setNum(config.buffer_watershed));
+	cpu_buffer_size_edit->setText(QString().setNum(config.cpu_buffer_size));
 	note_table_size_edit->setText(QString().setNum(config.note_table_size));
+	callgraph_depth_edit->setText(QString().setNum(config.callgraph_depth));
 	verbose->setChecked(config.verbose);
 	separate_lib_cb->setChecked(config.separate_lib);
 	separate_kernel_cb->setChecked(config.separate_kernel);
@@ -166,6 +178,12 @@ oprof_start::oprof_start()
 	buffer_size_edit->setValidator(iv);
 	iv = new QIntValidator(OP_MIN_NOTE_TABLE_SIZE, OP_MAX_NOTE_TABLE_SIZE, note_table_size_edit);
 	note_table_size_edit->setValidator(iv);
+	iv = new QIntValidator(0, INT_MAX, callgraph_depth_edit);
+	callgraph_depth_edit->setValidator(iv);
+	iv = new QIntValidator(0, INT_MAX, buffer_watershed_edit);
+	buffer_watershed_edit->setValidator(iv);
+	iv = new QIntValidator(0, OP_MAX_CPU_BUF_SIZE, cpu_buffer_size_edit);
+	cpu_buffer_size_edit->setValidator(iv);
 
 	// daemon status timer
 	startTimer(5000);
@@ -255,9 +273,8 @@ QListViewItem * findItem(QListView * view, char const * name)
 	// Qt 2.3.1 does not have QListView::findItem()
 	QListViewItem * item = view->firstChild();
 
-	while (item && strcmp(item->text(0).latin1(), name)) {
+	while (item && strcmp(item->text(0).latin1(), name))
 		item = item->nextSibling();
-	}
 
 	return item;
 }
@@ -283,7 +300,7 @@ void oprof_start::setup_default_event()
 
 void oprof_start::read_set_events()
 {
-	string name = get_user_filename(".oprofile/daemonrc");
+	string name = get_config_filename(".oprofile/daemonrc");
 
 	ifstream in(name.c_str());
 
@@ -300,12 +317,12 @@ void oprof_start::read_set_events()
 		string const val = split(str, '=');
 		string const name = str;
 
-		if (!is_prefix(name, "CHOSEN_EVENTS["))
+		if (!is_prefix(name, "CHOSEN_EVENTS_"))
 			continue;
 
 		one_enabled = true;
 
-		// CHOSEN_EVENTS[0]=CPU_CLK_UNHALTED:10000:0:1:1
+		// CHOSEN_EVENTS_#nr=CPU_CLK_UNHALTED:10000:0:1:1
 		vector<string> parts = separate_token(val, ':');
 
 		if (parts.size() != 5 && parts.size() != 2) {
@@ -345,7 +362,7 @@ void oprof_start::read_set_events()
 
 void oprof_start::load_config_file()
 {
-	string name = get_user_filename(".oprofile/daemonrc");
+	string name = get_config_filename(".oprofile/daemonrc");
 
 	ifstream in(name.c_str());
 	if (!in) {
@@ -391,6 +408,7 @@ void oprof_start::timerEvent(QTimerEvent *)
 	flush_profiler_data_btn->setEnabled(dstat.running);
 	stop_profiler_btn->setEnabled(dstat.running);
 	start_profiler_btn->setEnabled(!dstat.running);
+	reset_sample_files_btn->setEnabled(!dstat.running);
 
 	if (!dstat.running) {
 		daemon_label->setText("Profiler is not running.");
@@ -417,7 +435,7 @@ void oprof_start::fill_events_listbox()
 	setUpdatesEnabled(false);
 
 	for (vector<op_event_descr>::reverse_iterator cit = v_events.rbegin();
-		cit != v_events.rend(); ++cit) {
+	     cit != v_events.rend(); ++cit) {
 		new QListViewItem(events_list, cit->name.c_str());
 	}
 
@@ -483,9 +501,8 @@ bool oprof_start::alloc_selected_events() const
 	vector<op_event const *> events;
 
 	set<QListViewItem *>::const_iterator it;
-	for (it = selected_events.begin(); it != selected_events.end(); ++it) {
+	for (it = selected_events.begin(); it != selected_events.end(); ++it)
 		events.push_back(find_event_by_name((*it)->text(0).latin1()));
-	}
 
 	size_t * map =
 		map_event_to_counter(&events[0], events.size(), cpu_type);
@@ -506,9 +523,8 @@ void oprof_start::event_selected()
 	set<QListViewItem *> current_selection;
 	QListViewItem * cur;
 	for (cur = events_list->firstChild(); cur; cur = cur->nextSibling()) {
-		if (cur->isSelected()) {
+		if (cur->isSelected())
 			current_selection.insert(cur);
-		}
 	}
 
 	// First remove the deselected item.
@@ -517,9 +533,8 @@ void oprof_start::event_selected()
 		       current_selection.begin(), current_selection.end(),
 		       back_inserter(new_deselected));
 	vector<QListViewItem *>::const_iterator it;
-	for (it = new_deselected.begin(); it != new_deselected.end(); ++it) {
+	for (it = new_deselected.begin(); it != new_deselected.end(); ++it)
 		selected_events.erase(*it);
-	}
 
 	// Now try to add the newly selected item if enough HW resource exists
 	vector<QListViewItem *> new_selected;
@@ -605,8 +620,7 @@ bool oprof_start::record_config()
 	config.kernel_filename = kernel_filename_edit->text().latin1();
 	config.no_kernel = no_vmlinux->isChecked();
 
-	QString const t = buffer_size_edit->text();
-	uint temp = t.toUInt();
+	uint temp = buffer_size_edit->text().toUInt();
 	if (temp < OP_MIN_BUF_SIZE || temp > OP_MAX_BUF_SIZE) {
 		ostringstream error;
 
@@ -620,6 +634,36 @@ bool oprof_start::record_config()
 	}
 	config.buffer_size = temp;
 
+	temp = buffer_watershed_edit->text().toUInt();
+	// watershed above half of buffer size make little sense.
+	if (temp > config.buffer_size / 2) {
+		ostringstream error;
+
+		error << "buffer watershed out of range: " << temp
+		      << " valid range is [0 (use default), buffer size/2] "
+		      << "generally 0.25 * buffer size is fine";
+
+		QMessageBox::warning(this, 0, error.str().c_str());
+
+		return false;
+	}
+	config.buffer_watershed = temp;
+
+	temp = cpu_buffer_size_edit->text().toUInt();
+	if ((temp != 0 && temp < OP_MIN_CPU_BUF_SIZE) ||
+	    temp > OP_MAX_CPU_BUF_SIZE) {
+		ostringstream error;
+
+		error << "cpu buffer size out of range: " << temp
+		      << " valid range is [" << OP_MIN_CPU_BUF_SIZE << ", "
+		      << OP_MAX_CPU_BUF_SIZE << "] (size = 0: use default)";
+
+		QMessageBox::warning(this, 0, error.str().c_str());
+
+		return false;
+	}
+	config.cpu_buffer_size = temp;
+
 	temp = note_table_size_edit->text().toUInt();
 	if (temp < OP_MIN_NOTE_TABLE_SIZE || temp > OP_MAX_NOTE_TABLE_SIZE) {
 		ostringstream error;
@@ -634,6 +678,20 @@ bool oprof_start::record_config()
 	}
 	config.note_table_size = temp;
 
+	temp = callgraph_depth_edit->text().toUInt();
+	if (temp > INT_MAX) {
+		ostringstream error;
+
+		error << "callgraph depth  out of range: " << temp
+		      << " valid range is [" << 0 << ", "
+		      << INT_MAX << "]";
+
+		QMessageBox::warning(this, 0, error.str().c_str());
+
+		return false;
+	}
+	config.callgraph_depth = temp;
+
 	config.verbose = verbose->isChecked();
 	config.separate_lib = separate_lib_cb->isChecked();
 	config.separate_kernel = separate_kernel_cb->isChecked();
@@ -748,18 +806,11 @@ void oprof_start::setup_unit_masks(op_event_descr const & descr)
 			case 15: check = check15; break;
 		}
 		check->setText(um->um[i].desc);
-		if (um->unit_type_mask == utm_exclusive) {
+		if (um->unit_type_mask == utm_exclusive)
 			check->setChecked(cfg.umask == um->um[i].value);
-		} else {
-			// The last descriptor contains a mask that enable all
-			// value so we must enable the last check box only if
-			// all bits are on.
-			if (i == um->num - 1) {
-				check->setChecked(cfg.umask == um->um[i].value);
-			} else {
-				check->setChecked(cfg.umask & um->um[i].value);
-			}
-		}
+		else
+			check->setChecked(cfg.umask & um->um[i].value);
+
 		check->show();
 	}
 	unit_mask_group->setMinimumSize(unit_mask_group->sizeHint());
@@ -926,11 +977,21 @@ bool oprof_start::save_config()
 		args.push_back("--vmlinux=" + config.kernel_filename);
 	}
 
+	args.push_back("--buffer-size=" +
+	       op_lexical_cast<string>(config.buffer_size));
+
 	if (op_get_interface() == OP_INTERFACE_24) {
-		args.push_back("--buffer-size=" +
-		       op_lexical_cast<string>(config.buffer_size));
 		args.push_back("--note-table-size=" +
 		       op_lexical_cast<string>(config.note_table_size));
+	} else {
+		args.push_back("--buffer-watershed=" +
+		       op_lexical_cast<string>(config.buffer_watershed));
+		args.push_back("--cpu-buffer-size=" +
+		       op_lexical_cast<string>(config.cpu_buffer_size));
+		if (op_file_readable("/dev/oprofile/backtrace_depth")) {
+			args.push_back("--callgraph=" +
+		              op_lexical_cast<string>(config.callgraph_depth));
+		}
 	}
 
 	string sep = "--separate=";
@@ -975,6 +1036,22 @@ void oprof_start::on_separate_kernel_cb_changed(int state)
 		separate_lib_cb->setChecked(true);
 }
 
+void oprof_start::on_reset_sample_files()
+{
+	int ret = QMessageBox::warning(this, 0, "Are you sure you want to "
+	       "reset your last profile session ?", "Yes", "No", 0, 0, 1);
+	if (!ret) {
+		vector<string> args;
+		args.push_back("--reset");
+		if (!do_exec_command(OP_BINDIR "/opcontrol", args))
+			// the next timer event will overwrite the message
+			daemon_label->setText("Last profile session reseted.");
+		else
+			QMessageBox::warning(this, 0,
+			     "Can't reset profiling session.");
+	}
+}
+
 
 /// function object for matching against name
 class event_name_eq {
