@@ -42,7 +42,10 @@ config_setting::config_setting()
 	separate_lib(false),
 	separate_kernel(false),
 	separate_cpu(false),
-	separate_thread(false)
+	separate_thread(false),
+	callgraph_depth(0),
+	buffer_watershed(0),
+	cpu_buffer_size(0)
 {
 	struct utsname info;
 
@@ -91,6 +94,12 @@ void config_setting::load(istream & in)
 			separate_cpu = op_lexical_cast<bool>(val);
 		} else if (str == "SEPARATE_THREAD") {
 			separate_thread = op_lexical_cast<bool>(val);
+		} else if (str == "CALLGRAPH") {
+			callgraph_depth = op_lexical_cast<unsigned int>(val);
+		} else if (str == "BUF_WATERSHED") {
+			buffer_watershed = op_lexical_cast<unsigned int>(val);
+		} else if (str == "CPU_BUF_SIZE") {
+			cpu_buffer_size = op_lexical_cast<unsigned int>(val);
 		}
 	}
 }
