@@ -13,35 +13,20 @@
 #include <string.h>
 
 #include "op_cpu_type.h"
+#include "op_file.h"
 
-/**
- * op_get_interface - determine which kernel interface used
- *
- * returns %OP_INTERFACE_NO_GOOD if the interface could not be identified
- */
 op_interface op_get_interface(void)
 {
 	static op_interface current_interface = OP_INTERFACE_NO_GOOD;
-	FILE * fp;
 
 	if (current_interface != OP_INTERFACE_NO_GOOD)
 		return current_interface;
 
-	/* Try 2.4's interface. */
-	fp = fopen("/proc/sys/dev/oprofile/cpu_type", "r");
-	if (fp) {
-		fclose (fp);
+	if (op_file_readable("/proc/sys/dev/oprofile/cpu_type")) {
 		current_interface = OP_INTERFACE_24;
-		return current_interface;
+	} else if (op_file_readable("/dev/oprofile/cpu_type")) {
+		current_interface = OP_INTERFACE_26;
 	}
 
-	/* Try 2.5's oprofilefs one instead. */
-	fp = fopen("/dev/oprofile/cpu_type", "r");
-	if (fp) {
-		fclose (fp);
-		current_interface = OP_INTERFACE_25;
-		return current_interface;
-	}
-
-	return OP_INTERFACE_NO_GOOD;
+	return current_interface;
 }
