@@ -12,13 +12,21 @@
 
 #include "op_bfd.h"
 #include "op_fileio.h"
+#include "op_config.h"
 #include "string_manip.h"
+#include "file_manip.h"
 #include "cverb.h"
+#include "locate_images.h"
+
+#include <cstdlib>
+#include <cstring>
 
 #include <iostream>
 #include <fstream>
 #include <sstream>
 #include <string>
+#include <cstring>
+#include <cstdlib>
 
 using namespace std;
 
@@ -37,13 +45,22 @@ void check_format(string const & file, bfd ** ibfd)
 }
 
 
-bool separate_debug_file_exists(string const & name, unsigned long const crc)
+bool separate_debug_file_exists(string & name, unsigned long const crc, 
+                                extra_images const & extra)
 {
 	unsigned long file_crc = 0;
 	// The size of 2 * 1024 elements for the buffer is arbitrary.
 	char buffer[2 * 1024];
-	
-	ifstream file(name.c_str());
+
+	image_error img_ok;
+	string const image_path = extra.find_image_path(name, img_ok, true);
+
+	if (img_ok != image_ok)
+		return false;
+
+	name = image_path;
+
+	ifstream file(image_path.c_str());
 	if (!file)
 		return false;
 
@@ -74,9 +91,12 @@ bool get_debug_link_info(bfd * ibfd, string & filename, unsigned long & crc32)
 	cverb << vbfd
 	      << ".gnu_debuglink section has size " << debuglink_size << endl;
 	
-	bfd_get_section_contents(ibfd, sect, 
+	if (!bfd_get_section_contents(ibfd, sect, 
 				 reinterpret_cast<unsigned char *>(contents), 
-				 static_cast<file_ptr>(0), debuglink_size);
+				 static_cast<file_ptr>(0), debuglink_size)) {
+		bfd_perror("bfd_get_section_contents:get_debug:");
+		exit(2);
+	}
 	
 	/* CRC value is stored after the filename, aligned up to 4 bytes. */
 	size_t filename_len = strlen(contents);
@@ -273,40 +293,35 @@ bfd * fdopen_bfd(string const & file, int fd)
 }
 
 
-bool find_separate_debug_file(bfd * ibfd, string const & dir_in,
-                              string const & global_in, string & filename)
+bool find_separate_debug_file(bfd * ibfd, string const & filepath_in, 
+                              string & debug_filename, extra_images const & extra)
 {
-	string dir(dir_in);
-	string global(global_in);
+	string filepath(filepath_in);
 	string basename;
 	unsigned long crc32;
 	
 	if (!get_debug_link_info(ibfd, basename, crc32))
 		return false;
-	
-	if (dir.size() > 0 && dir.at(dir.size() - 1) != '/')
-		dir += '/';
-	
-	if (global.size() > 0 && global.at(global.size() - 1) != '/')
-		global += '/';
+
+	// Work out the image file's directory prefix
+	string filedir = op_dirname(filepath);
+	// Make sure it starts with /
+	if (filedir.size() > 0 && filedir.at(filedir.size() - 1) != '/')
+		filedir += '/';
+
+	string first_try(filedir + ".debug/" + basename);
+	string second_try(DEBUGDIR + filedir + basename);
+	string third_try(filedir + basename);
 
 	cverb << vbfd << "looking for debugging file " << basename 
 	      << " with crc32 = " << hex << crc32 << endl;
-	
-	string first_try(dir + basename);
-	string second_try(dir + ".debug/" + basename);
 
-	if (dir.size() > 0 && dir[0] == '/')
-		dir = dir.substr(1);
-
-	string third_try(global + dir + basename);
-	
-	if (separate_debug_file_exists(first_try, crc32)) 
-		filename = first_try; 
-	else if (separate_debug_file_exists(second_try, crc32))
-		filename = second_try;
-	else if (separate_debug_file_exists(third_try, crc32))
-		filename = third_try;
+	if (separate_debug_file_exists(first_try, crc32, extra)) 
+		debug_filename = first_try; 
+	else if (separate_debug_file_exists(second_try, crc32, extra))
+		debug_filename = second_try;
+	else if (separate_debug_file_exists(third_try, crc32, extra))
+		debug_filename = third_try;
 	else
 		return false;
 	
@@ -331,6 +346,11 @@ bool interesting_symbol(asymbol * sym)
 	// returning true for fix up in op_bfd_symbol()
 	if (!sym->name || sym->name[0] == '\0')
 		return true;
+	/* ARM assembler internal mapping symbols aren't interesting */
+	if ((strcmp("$a", sym->name) == 0) ||
+	    (strcmp("$t", sym->name) == 0) ||
+	    (strcmp("$d", sym->name) == 0))
+		return false;
 
 	// C++ exception stuff
 	if (sym->name[0] == '.' && sym->name[1] == 'L')
@@ -498,7 +518,7 @@ void bfd_info::get_symbols()
 
 linenr_info const
 find_nearest_line(bfd_info const & b, op_bfd_symbol const & sym,
-                  unsigned int offset)
+                  unsigned int offset, bool anon_obj)
 {
 	char const * function = "";
 	char const * cfilename = "";
@@ -520,7 +540,10 @@ find_nearest_line(bfd_info const & b, op_bfd_symbol const & sym,
 	abfd = b.abfd;
 	syms = b.syms.get();
 	section = sym.symbol()->section;
-	pc = (sym.value() + offset) - sym.filepos();
+	if (anon_obj)
+		pc = offset - sym.symbol()->section->vma;
+	else
+		pc = (sym.value() + offset) - sym.filepos();
 
 	if ((bfd_get_section_flags(abfd, section) & SEC_ALLOC) == 0)
 		goto fail;
@@ -534,7 +557,12 @@ find_nearest_line(bfd_info const & b, op_bfd_symbol const & sym,
 	if (!ret || !cfilename || !function)
 		goto fail;
 
-	if (!is_correct_function(function, sym.name()))
+	/*
+	 * is_correct_function does not handle the case of static inlines,
+	 * but if the linenr is non-zero in the inline case, it is the correct
+	 * line number.
+	 */
+	if (linenr == 0 && !is_correct_function(function, sym.name()))
 		goto fail;
 
 	if (linenr == 0) {
