@@ -13,8 +13,10 @@
 
 #include "utility.h"
 #include "op_types.h"
+#include "locate_images.h"
 
 #include <bfd.h>
+#include <stdint.h>
 
 #include <string>
 
@@ -83,9 +85,9 @@ private:
  */
 extern bool
 find_separate_debug_file(bfd * ibfd, 
-                         std::string const & dir_in,
-                         std::string const & global_in,
-                         std::string & filename);
+                         std::string const & filepath_in,
+                         std::string & debug_filename,
+                         extra_images const & extra);
 
 /// open the given BFD
 bfd * open_bfd(std::string const & file);
@@ -93,6 +95,9 @@ bfd * open_bfd(std::string const & file);
 /// open the given BFD from the fd
 bfd * fdopen_bfd(std::string const & file, int fd);
 
+/// Return a BFD for an SPU ELF embedded in PPE binary file
+bfd * spu_open_bfd(std::string const name, int fd, uint64_t offset_to_spu_elf);
+
 /// Return true if the symbol is worth looking at
 bool interesting_symbol(asymbol * sym);
 
@@ -124,6 +129,6 @@ struct linenr_info {
  */
 linenr_info const
 find_nearest_line(bfd_info const & ibfd, op_bfd_symbol const & sym,
-                  unsigned int offset);
+                  unsigned int offset, bool anon_obj);
 
 #endif /* !BFD_SUPPORT_H */
