@@ -18,10 +18,10 @@ class profile_classes;
 class merge_option;
 
 namespace options {
-	extern std::string archive_path;
 	extern bool exclude_dependent;
 	extern merge_option merge_by;
 	extern std::string outdirectory;
+	extern bool list_files;
 }
 
 /// All the chosen sample files.
